{% block title %}
InvenioILS: loan request registered for "{{ document.title|safe }}"
{% endblock %}

{% block body_plain %}
Dear {{ patron.name }},

your loan request for "{{ document.full_title }}" <{{ spa_routes.HOST }}{{ spa_routes.PATHS['literature']|format(pid=document.pid) }}> has been registered.
{% endblock %}

{% block body_html %}
Dear {{ patron.name }},<br/><br/>

your loan request for <a href="{{ spa_routes.HOST }}{{ spa_routes.PATHS['literature']|format(pid=document.pid) }}">"{{ document.full_title }}"</a> has been <b>registered</b>.<br/>
{% endblock %}
