{% block title %}
InvenioILS: due loan reminder for "{{ document.title|safe }}"
{% endblock %}

{% block body_plain %}
Dear {{ patron.name }},

your loan on "{{ document.full_title }}" <{{ spa_routes.HOST }}{{ spa_routes.PATHS['literature']|format(pid=document.pid) }}> is due in {{ expiring_in_days }} days.

Do not forget to bring back the borrowed literature in time.
{% endblock %}

{% block body_html %}
Dear {{ patron.name }},<br/><br/>

your loan on <a href="{{ spa_routes.HOST }}{{ spa_routes.PATHS['literature']|format(pid=document.pid) }}">"{{ document.full_title }}"</a> is due in {{ expiring_in_days }} days.<br/><br/>

Do not forget to bring back the borrowed literature in time.<br/>
{% endblock %}
